@@ -96,8 +96,7 @@ typedef struct _GRealArray  GRealArray;
  * @len: the number of elements in the #GArray not including the
  *       possible terminating zero element.
  *
- * Contains the public fields of an <link
- * linkend="glib-arrays">Array</link>.
+ * Contains the public fields of an <link linkend="glib-Arrays">Array</link>.
  **/
 struct _GRealArray
 {
@@ -107,7 +106,7 @@ struct _GRealArray
   guint   elt_size;
   guint   zero_terminated : 1;
   guint   clear : 1;
-  volatile gint ref_count;
+  gint    ref_count;
 };
 
 /**
@@ -617,7 +616,10 @@ g_array_remove_range (GArray *farray,
  * greater than second arg).
  *
  * If two array elements compare equal, their order in the sorted array
- * is undefined.
+ * is undefined. If you want equal elements to keep their order &#8211; i.e.
+ * you want a stable sort &#8211; you can write a comparison function that,
+ * if two elements would otherwise compare equal, compares them by
+ * their addresses.
  **/
 void
 g_array_sort (GArray       *farray,
@@ -756,7 +758,7 @@ struct _GRealPtrArray
   gpointer     *pdata;
   guint         len;
   guint         alloc;
-  volatile gint ref_count;
+  gint          ref_count;
   GDestroyNotify element_free_func;
 };
 
@@ -1239,7 +1241,10 @@ g_ptr_array_add (GPtrArray *farray,
  * greater than second arg).
  *
  * If two array elements compare equal, their order in the sorted array
- * is undefined.
+ * is undefined. If you want equal elements to keep their order &#8211; i.e.
+ * you want a stable sort &#8211; you can write a comparison function that,
+ * if two elements would otherwise compare equal, compares them by
+ * their addresses.
  *
  * <note><para>The comparison function for g_ptr_array_sort() doesn't
  * take the pointers from the array as arguments, it takes pointers to
@@ -1561,7 +1566,10 @@ g_byte_array_remove_range (GByteArray *array,
  * first arg is greater than second arg).
  *
  * If two array elements compare equal, their order in the sorted array
- * is undefined.
+ * is undefined. If you want equal elements to keep their order &#8211; i.e.
+ * you want a stable sort &#8211; you can write a comparison function that,
+ * if two elements would otherwise compare equal, compares them by
+ * their addresses.
  **/
 void
 g_byte_array_sort (GByteArray   *array,
