@@ -769,13 +769,14 @@ g_convert_with_iconv (const gchar *str,
 	      have_error = TRUE;
 	      break;
 	    default:
-              {
-                int errsv = errno;
-                
-                g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
-                             _("Error during conversion: %s"),
-                             g_strerror (errsv));
-              }
+	      if (error)
+                {
+                  int errsv = errno;
+
+                  g_set_error (error, G_CONVERT_ERROR, G_CONVERT_ERROR_FAILED,
+                               _("Error during conversion: %s"),
+                               g_strerror (errsv));
+                }
 	      have_error = TRUE;
 	      break;
 	    }
@@ -836,7 +837,7 @@ g_convert_with_iconv (const gchar *str,
                    </footnote>. 
  * @to_codeset:    name of character set into which to convert @str
  * @from_codeset:  character set of @str.
- * @bytes_read:    location to store the number of bytes in the
+ * @bytes_read: (out):   location to store the number of bytes in the
  *                 input string that were successfully converted, or %NULL.
  *                 Even if the conversion was successful, this may be 
  *                 less than @len if there were partial characters
@@ -844,7 +845,7 @@ g_convert_with_iconv (const gchar *str,
  *                 #G_CONVERT_ERROR_ILLEGAL_SEQUENCE occurs, the value
  *                 stored will the byte offset after the last valid
  *                 input sequence.
- * @bytes_written: the number of bytes stored in the output buffer (not 
+ * @bytes_written: (out): the number of bytes stored in the output buffer (not 
  *                 including the terminating nul).
  * @error:         location to store the error occuring, or %NULL to ignore
  *                 errors. Any of the errors in #GConvertError may occur.
@@ -1317,7 +1318,7 @@ filename_charset_cache_free (gpointer data)
  * Since: 2.6
  */
 gboolean
-g_get_filename_charsets (G_CONST_RETURN gchar ***filename_charsets)
+g_get_filename_charsets (const gchar ***filename_charsets)
 {
   static GStaticPrivate cache_private = G_STATIC_PRIVATE_INIT;
   GFilenameCharsetCache *cache = g_static_private_get (&cache_private);
@@ -1382,7 +1383,7 @@ g_get_filename_charsets (G_CONST_RETURN gchar ***filename_charsets)
 #else /* G_PLATFORM_WIN32 */
 
 gboolean
-g_get_filename_charsets (G_CONST_RETURN gchar ***filename_charsets) 
+g_get_filename_charsets (const gchar ***filename_charsets) 
 {
   static const gchar *charsets[] = {
     "UTF-8",
@@ -2006,7 +2007,7 @@ g_filename_from_uri (const gchar *uri,
  * @filename: an absolute filename specified in the GLib file name encoding,
  *            which is the on-disk file name bytes on Unix, and UTF-8 on 
  *            Windows
- * @hostname: A UTF-8 encoded hostname, or %NULL for none.
+ * @hostname: (allow-none): A UTF-8 encoded hostname, or %NULL for none.
  * @error: location to store the error occuring, or %NULL to ignore
  *         errors. Any of the errors in #GConvertError may occur.
  * 
@@ -2160,7 +2161,7 @@ g_uri_list_extract_uris (const gchar *uri_list)
  * for instance there might be problems converting it to UTF-8, and some files
  * can be translated in the display.
  *
- * If GLib can not make sense of the encoding of @filename, as a last resort it 
+ * If GLib cannot make sense of the encoding of @filename, as a last resort it 
  * replaces unknown characters with U+FFFD, the Unicode replacement character.
  * You can search the result for the UTF-8 encoding of this character (which is
  * "\357\277\275" in octal notation) to find out if @filename was in an invalid
@@ -2201,7 +2202,7 @@ g_filename_display_basename (const gchar *filename)
  * Unlike g_filename_to_utf8(), the result is guaranteed to be non-%NULL 
  * even if the filename actually isn't in the GLib file name encoding.
  *
- * If GLib can not make sense of the encoding of @filename, as a last resort it 
+ * If GLib cannot make sense of the encoding of @filename, as a last resort it 
  * replaces unknown characters with U+FFFD, the Unicode replacement character.
  * You can search the result for the UTF-8 encoding of this character (which is
  * "\357\277\275" in octal notation) to find out if @filename was in an invalid
