@@ -200,8 +200,10 @@
 
 #define HASH_TABLE_MIN_SHIFT 3  /* 1 << 3 == 8 buckets */
 
-#define HASH_IS_UNUSED(h_) ((h_) == 0)
-#define HASH_IS_TOOMBSTONE(h_) ((h_) == 1)
+#define UNUSED_HASH_VALUE 0
+#define TOMBSTONE_HASH_VALUE 1
+#define HASH_IS_UNUSED(h_) ((h_) == UNUSED_HASH_VALUE)
+#define HASH_IS_TOMBSTONE(h_) ((h_) == TOMBSTONE_HASH_VALUE)
 #define HASH_IS_REAL(h_) ((h_) >= 2)
 
 struct _GHashTable
@@ -218,7 +220,7 @@ struct _GHashTable
 
   GHashFunc        hash_func;
   GEqualFunc       key_equal_func;
-  volatile gint    ref_count;
+  gint             ref_count;
 #ifndef G_DISABLE_ASSERT
   /*
    * Tracks the structure of the hash table, not its contents: is only
@@ -345,35 +347,34 @@ g_hash_table_set_shift_from_size (GHashTable *hash_table, gint size)
  */
 static inline guint
 g_hash_table_lookup_node (GHashTable    *hash_table,
-			  gconstpointer  key,
-			  guint         *hash_return)
+                          gconstpointer  key,
+                          guint         *hash_return)
 {
   guint node_index;
+  guint node_hash;
   guint hash_value;
   guint first_tombstone = 0;
   gboolean have_tombstone = FALSE;
   guint step = 0;
 
-  hash_value = (* hash_table->hash_func) (key);
+  hash_value = hash_table->hash_func (key);
   if (G_UNLIKELY (!HASH_IS_REAL (hash_value)))
     hash_value = 2;
 
   *hash_return = hash_value;
 
   node_index = hash_value % hash_table->mod;
+  node_hash = hash_table->hashes[node_index];
 
-  while (!HASH_IS_UNUSED (hash_table->hashes[node_index]))
+  while (!HASH_IS_UNUSED (node_hash))
     {
-      guint node_hash = hash_table->hashes[node_index];
-
-      /*  We first check if our full hash values
-       *  are equal so we can avoid calling the full-blown
-       *  key equality function in most cases.
+      /* We first check if our full hash values
+       * are equal so we can avoid calling the full-blown
+       * key equality function in most cases.
        */
-
       if (node_hash == hash_value)
         {
-	  gpointer node_key = hash_table->keys[node_index];
+          gpointer node_key = hash_table->keys[node_index];
 
           if (hash_table->key_equal_func)
             {
@@ -385,7 +386,7 @@ g_hash_table_lookup_node (GHashTable    *hash_table,
               return node_index;
             }
         }
-      else if (HASH_IS_TOOMBSTONE (node_hash) && !have_tombstone)
+      else if (HASH_IS_TOMBSTONE (node_hash) && !have_tombstone)
         {
           first_tombstone = node_index;
           have_tombstone = TRUE;
@@ -394,6 +395,7 @@ g_hash_table_lookup_node (GHashTable    *hash_table,
       step++;
       node_index += step;
       node_index &= hash_table->mask;
+      node_hash = hash_table->hashes[node_index];
     }
 
   if (have_tombstone)
@@ -419,20 +421,27 @@ g_hash_table_remove_node (GHashTable   *hash_table,
                           int           i,
                           gboolean      notify)
 {
-  if (notify && hash_table->key_destroy_func)
-    hash_table->key_destroy_func (hash_table->keys[i]);
+  gpointer key;
+  gpointer value;
 
-  if (notify && hash_table->value_destroy_func)
-    hash_table->value_destroy_func (hash_table->values[i]);
+  key = hash_table->keys[i];
+  value = hash_table->values[i];
 
   /* Erect tombstone */
-  hash_table->hashes[i] = 1;
+  hash_table->hashes[i] = TOMBSTONE_HASH_VALUE;
 
   /* Be GC friendly */
   hash_table->keys[i] = NULL;
   hash_table->values[i] = NULL;
 
   hash_table->nnodes--;
+
+  if (notify && hash_table->key_destroy_func)
+    hash_table->key_destroy_func (key);
+
+  if (notify && hash_table->value_destroy_func)
+    hash_table->value_destroy_func (value);
+
 }
 
 /*
@@ -451,33 +460,45 @@ g_hash_table_remove_all_nodes (GHashTable *hash_table,
                                gboolean    notify)
 {
   int i;
-
-  if (notify &&
-      (hash_table->key_destroy_func != NULL ||
-       hash_table->value_destroy_func != NULL))
-    {
-      for (i = 0; i < hash_table->size; i++)
-        {
-          if (HASH_IS_REAL (hash_table->hashes[i]))
-            {
-              if (hash_table->key_destroy_func != NULL)
-                hash_table->key_destroy_func (hash_table->keys[i]);
-
-              if (hash_table->value_destroy_func != NULL)
-                hash_table->value_destroy_func (hash_table->values[i]);
-            }
-        }
-    }
-
-  /* We need to set node->key_hash = 0 for all nodes - might as well be GC
-   * friendly and clear everything
-   */
-  memset (hash_table->hashes, 0, hash_table->size * sizeof (guint));
-  memset (hash_table->keys, 0, hash_table->size * sizeof (gpointer));
-  memset (hash_table->values, 0, hash_table->size * sizeof (gpointer));
+  gpointer key;
+  gpointer value;
 
   hash_table->nnodes = 0;
   hash_table->noccupied = 0;
+
+  if (!notify ||
+      (hash_table->key_destroy_func == NULL &&
+       hash_table->value_destroy_func == NULL))
+    {
+      memset (hash_table->hashes, 0, hash_table->size * sizeof (guint));
+      memset (hash_table->keys, 0, hash_table->size * sizeof (gpointer));
+      memset (hash_table->values, 0, hash_table->size * sizeof (gpointer));
+
+      return;
+    }
+
+  for (i = 0; i < hash_table->size; i++)
+    {
+      if (HASH_IS_REAL (hash_table->hashes[i]))
+        {
+          key = hash_table->keys[i];
+          value = hash_table->values[i];
+
+          hash_table->hashes[i] = UNUSED_HASH_VALUE;
+          hash_table->keys[i] = NULL;
+          hash_table->values[i] = NULL;
+
+          if (hash_table->key_destroy_func != NULL)
+            hash_table->key_destroy_func (key);
+
+          if (hash_table->value_destroy_func != NULL)
+            hash_table->value_destroy_func (value);
+        }
+      else if (HASH_IS_TOMBSTONE (hash_table->hashes[i]))
+        {
+          hash_table->hashes[i] = UNUSED_HASH_VALUE;
+        }
+    }
 }
 
 /*
@@ -662,7 +683,7 @@ g_hash_table_new_full (GHashFunc       hash_func,
  **/
 void
 g_hash_table_iter_init (GHashTableIter *iter,
-			GHashTable     *hash_table)
+                        GHashTable     *hash_table)
 {
   RealIter *ri = (RealIter *) iter;
 
@@ -692,8 +713,8 @@ g_hash_table_iter_init (GHashTableIter *iter,
  **/
 gboolean
 g_hash_table_iter_next (GHashTableIter *iter,
-			gpointer       *key,
-			gpointer       *value)
+                        gpointer       *key,
+                        gpointer       *value)
 {
   RealIter *ri = (RealIter *) iter;
   gint position;
@@ -818,9 +839,9 @@ GHashTable*
 g_hash_table_ref (GHashTable *hash_table)
 {
   g_return_val_if_fail (hash_table != NULL, NULL);
-  g_return_val_if_fail (hash_table->ref_count > 0, hash_table);
 
-  g_atomic_int_add (&hash_table->ref_count, 1);
+  g_atomic_int_inc (&hash_table->ref_count);
+
   return hash_table;
 }
 
@@ -839,9 +860,8 @@ void
 g_hash_table_unref (GHashTable *hash_table)
 {
   g_return_if_fail (hash_table != NULL);
-  g_return_if_fail (hash_table->ref_count > 0);
 
-  if (g_atomic_int_exchange_and_add (&hash_table->ref_count, -1) - 1 == 0)
+  if (g_atomic_int_dec_and_test (&hash_table->ref_count))
     {
       g_hash_table_remove_all_nodes (hash_table, TRUE);
       if (hash_table->keys != hash_table->values)
@@ -867,7 +887,6 @@ void
 g_hash_table_destroy (GHashTable *hash_table)
 {
   g_return_if_fail (hash_table != NULL);
-  g_return_if_fail (hash_table->ref_count > 0);
 
   g_hash_table_remove_all (hash_table);
   g_hash_table_unref (hash_table);
@@ -969,9 +988,10 @@ g_hash_table_insert_internal (GHashTable *hash_table,
   guint node_index;
   guint key_hash;
   guint old_hash;
+  gpointer old_key;
+  gpointer old_value;
 
   g_return_if_fail (hash_table != NULL);
-  g_return_if_fail (hash_table->ref_count > 0);
 
   if (G_UNLIKELY (hash_table->keys == hash_table->values && key != value))
     hash_table->values = g_memdup (hash_table->keys, sizeof (gpointer) * hash_table->size);
@@ -979,24 +999,13 @@ g_hash_table_insert_internal (GHashTable *hash_table,
   node_index = g_hash_table_lookup_node (hash_table, key, &key_hash);
 
   old_hash = hash_table->hashes[node_index];
+  old_key = hash_table->keys[node_index];
+  old_value = hash_table->values[node_index];
 
   if (HASH_IS_REAL (old_hash))
     {
-      if (hash_table->value_destroy_func)
-        hash_table->value_destroy_func (hash_table->values[node_index]);
-
       if (keep_new_key)
-        {
-          if (hash_table->key_destroy_func)
-            hash_table->key_destroy_func (hash_table->keys[node_index]);
-          hash_table->keys[node_index] = key;
-        }
-      else
-        {
-          if (hash_table->key_destroy_func)
-            hash_table->key_destroy_func (key);
-        }
-
+        hash_table->keys[node_index] = key;
       hash_table->values[node_index] = value;
     }
   else
@@ -1018,6 +1027,14 @@ g_hash_table_insert_internal (GHashTable *hash_table,
       hash_table->version++;
 #endif
     }
+
+  if (HASH_IS_REAL (old_hash))
+    {
+      if (hash_table->key_destroy_func)
+        hash_table->key_destroy_func (keep_new_key ? old_key : key);
+      if (hash_table->value_destroy_func)
+        hash_table->value_destroy_func (old_value);
+    }
 }
 
 /**
@@ -1208,12 +1225,15 @@ g_hash_table_steal_all (GHashTable *hash_table)
  */
 static guint
 g_hash_table_foreach_remove_or_steal (GHashTable *hash_table,
-                                      GHRFunc	  func,
+                                      GHRFunc     func,
                                       gpointer    user_data,
                                       gboolean    notify)
 {
   guint deleted = 0;
   gint i;
+#ifndef G_DISABLE_ASSERT
+  gint version = hash_table->version;
+#endif
 
   for (i = 0; i < hash_table->size; i++)
     {
@@ -1222,11 +1242,15 @@ g_hash_table_foreach_remove_or_steal (GHashTable *hash_table,
       gpointer node_value = hash_table->values[i];
 
       if (HASH_IS_REAL (node_hash) &&
-	  (* func) (node_key, node_value, user_data))
+          (* func) (node_key, node_value, user_data))
         {
           g_hash_table_remove_node (hash_table, i, notify);
           deleted++;
         }
+
+#ifndef G_DISABLE_ASSERT
+      g_return_val_if_fail (version == hash_table->version, 0);
+#endif
     }
 
   g_hash_table_maybe_resize (hash_table);
@@ -1277,7 +1301,7 @@ g_hash_table_foreach_remove (GHashTable *hash_table,
  * If the function returns %TRUE, then the key/value pair is removed from the
  * #GHashTable, but no key or value destroy functions are called.
  *
- * See #GHashTableIter for an alternative way to loop over the 
+ * See #GHashTableIter for an alternative way to loop over the
  * key/value pairs in the hash table.
  *
  * Return value: the number of key/value pairs removed.
@@ -1315,6 +1339,9 @@ g_hash_table_foreach (GHashTable *hash_table,
                       gpointer    user_data)
 {
   gint i;
+#ifndef G_DISABLE_ASSERT
+  gint version = hash_table->version;
+#endif
 
   g_return_if_fail (hash_table != NULL);
   g_return_if_fail (func != NULL);
@@ -1327,6 +1354,10 @@ g_hash_table_foreach (GHashTable *hash_table,
 
       if (HASH_IS_REAL (node_hash))
         (* func) (node_key, node_value, user_data);
+
+#ifndef G_DISABLE_ASSERT
+      g_return_if_fail (version == hash_table->version);
+#endif
     }
 }
 
@@ -1350,30 +1381,42 @@ g_hash_table_foreach (GHashTable *hash_table,
  * operation issued for all n values in a hash table ends up needing O(n*n)
  * operations).
  *
- * Return value: The value of the first key/value pair is returned, for which
- * func evaluates to %TRUE. If no pair with the requested property is found,
- * %NULL is returned.
+ * Return value: The value of the first key/value pair is returned,
+ *     for which @predicate evaluates to %TRUE. If no pair with the
+ *     requested property is found, %NULL is returned.
  *
  * Since: 2.4
  **/
 gpointer
-g_hash_table_find (GHashTable      *hash_table,
-                   GHRFunc          predicate,
-                   gpointer         user_data)
+g_hash_table_find (GHashTable *hash_table,
+                   GHRFunc     predicate,
+                   gpointer    user_data)
 {
   gint i;
+#ifndef G_DISABLE_ASSERT
+  gint version = hash_table->version;
+#endif
+  gboolean match;
 
   g_return_val_if_fail (hash_table != NULL, NULL);
   g_return_val_if_fail (predicate != NULL, NULL);
 
+  match = FALSE;
+
   for (i = 0; i < hash_table->size; i++)
     {
       guint node_hash = hash_table->hashes[i];
       gpointer node_key = hash_table->keys[i];
       gpointer node_value = hash_table->values[i];
 
-      if (HASH_IS_REAL (node_hash) &&
-	  predicate (node_key, node_value, user_data))
+      if (HASH_IS_REAL (node_hash))
+        match = predicate (node_key, node_value, user_data);
+
+#ifndef G_DISABLE_ASSERT
+      g_return_val_if_fail (version == hash_table->version, NULL);
+#endif
+
+      if (match)
         return node_value;
     }
 
