@@ -19,6 +19,10 @@
  *   Boston, MA 02111-1307, USA.
  */
 
+#if defined(G_DISABLE_SINGLE_INCLUDES) && !defined (__GLIB_H_INSIDE__) && !defined (GLIB_COMPILATION)
+#error "Only <glib.h> can be included directly."
+#endif
+
 #ifndef __G_UNICODE_H__
 #define __G_UNICODE_H__
 
@@ -109,7 +113,8 @@ typedef enum
   G_UNICODE_BREAK_HANGUL_V_JAMO,
   G_UNICODE_BREAK_HANGUL_T_JAMO,
   G_UNICODE_BREAK_HANGUL_LV_SYLLABLE,
-  G_UNICODE_BREAK_HANGUL_LVT_SYLLABLE
+  G_UNICODE_BREAK_HANGUL_LVT_SYLLABLE,
+  G_UNICODE_BREAK_CLOSE_PARANTHESIS
 } GUnicodeBreakType;
 
 typedef enum 
@@ -171,7 +176,7 @@ typedef enum
   G_UNICODE_SCRIPT_LINEAR_B,           /* Linb */
   G_UNICODE_SCRIPT_TAI_LE,             /* Tale */
   G_UNICODE_SCRIPT_UGARITIC,           /* Ugar */
-      
+
   /* Unicode-4.1 additions */
   G_UNICODE_SCRIPT_NEW_TAI_LUE,        /* Talu */
   G_UNICODE_SCRIPT_BUGINESE,           /* Bugi */
@@ -187,7 +192,42 @@ typedef enum
   G_UNICODE_SCRIPT_CUNEIFORM,          /* Xsux */
   G_UNICODE_SCRIPT_PHOENICIAN,         /* Phnx */
   G_UNICODE_SCRIPT_PHAGS_PA,           /* Phag */
-  G_UNICODE_SCRIPT_NKO                 /* Nkoo */
+  G_UNICODE_SCRIPT_NKO,                /* Nkoo */
+
+  /* Unicode-5.1 additions */
+  G_UNICODE_SCRIPT_KAYAH_LI,           /* Kali */
+  G_UNICODE_SCRIPT_LEPCHA,             /* Lepc */
+  G_UNICODE_SCRIPT_REJANG,             /* Rjng */
+  G_UNICODE_SCRIPT_SUNDANESE,          /* Sund */
+  G_UNICODE_SCRIPT_SAURASHTRA,         /* Saur */
+  G_UNICODE_SCRIPT_CHAM,               /* Cham */
+  G_UNICODE_SCRIPT_OL_CHIKI,           /* Olck */
+  G_UNICODE_SCRIPT_VAI,                /* Vaii */
+  G_UNICODE_SCRIPT_CARIAN,             /* Cari */
+  G_UNICODE_SCRIPT_LYCIAN,             /* Lyci */
+  G_UNICODE_SCRIPT_LYDIAN,             /* Lydi */
+
+  /* Unicode-5.2 additions */
+  G_UNICODE_SCRIPT_AVESTAN,                /* Avst */
+  G_UNICODE_SCRIPT_BAMUM,                  /* Bamu */
+  G_UNICODE_SCRIPT_EGYPTIAN_HIEROGLYPHS,   /* Egyp */
+  G_UNICODE_SCRIPT_IMPERIAL_ARAMAIC,       /* Armi */
+  G_UNICODE_SCRIPT_INSCRIPTIONAL_PAHLAVI,  /* Phli */
+  G_UNICODE_SCRIPT_INSCRIPTIONAL_PARTHIAN, /* Prti */
+  G_UNICODE_SCRIPT_JAVANESE,               /* Java */
+  G_UNICODE_SCRIPT_KAITHI,                 /* Kthi */
+  G_UNICODE_SCRIPT_LISU,                   /* Lisu */
+  G_UNICODE_SCRIPT_MEETEI_MAYEK,           /* Mtei */
+  G_UNICODE_SCRIPT_OLD_SOUTH_ARABIAN,      /* Sarb */
+  G_UNICODE_SCRIPT_OLD_TURKIC,             /* Orkh */
+  G_UNICODE_SCRIPT_SAMARITAN,              /* Samr */
+  G_UNICODE_SCRIPT_TAI_THAM,               /* Lana */
+  G_UNICODE_SCRIPT_TAI_VIET,               /* Tavt */
+
+  /* Unicode-6.0 additions */
+  G_UNICODE_SCRIPT_BATAK,                  /* Batk */
+  G_UNICODE_SCRIPT_BRAHMI,                 /* Brah */
+  G_UNICODE_SCRIPT_MANDAIC                 /* Mand */
 } GUnicodeScript;
 
 /* Returns TRUE if current locale uses UTF-8 charset.  If CHARSET is
@@ -196,7 +236,7 @@ typedef enum
  * in case the locale's charset will be changed later using setlocale()
  * or in some other way.
  */
-gboolean g_get_charset (G_CONST_RETURN char **charset);
+gboolean g_get_charset (const char **charset);
 
 /* These are all analogs of the <ctype.h> functions.
  */
@@ -236,6 +276,9 @@ GUnicodeType g_unichar_type (gunichar c) G_GNUC_CONST;
 /* Return the line break property for a given character */
 GUnicodeBreakType g_unichar_break_type (gunichar c) G_GNUC_CONST;
 
+/* Returns the combining class for a given character */
+gint g_unichar_combining_class (gunichar uc) G_GNUC_CONST;
+
 
 /* Compute canonical ordering of a string in-place.  This rearranges
    decomposed characters in the string according to their combining
@@ -379,7 +422,6 @@ GUnicodeScript g_unichar_get_script (gunichar ch) G_GNUC_CONST;
 
 gchar *_g_utf8_make_valid (const gchar *name);
 
-
 G_END_DECLS
 
 #endif /* __G_UNICODE_H__ */
