@@ -31,6 +31,7 @@
 #include "config.h"
 
 #include "glib.h"
+#include "gmodule.h"
 
 #include <errno.h>
 #include <string.h>
@@ -46,7 +47,6 @@
 
 #include "gmoduleconf.h"
 #include "gstdio.h"
-#include "gmodule.h"
 
 /* We maintain a list of modules, so we can reference count them.
  * That's needed because some platforms don't support refernce counts on
@@ -598,7 +598,7 @@ g_module_make_resident (GModule *module)
   module->is_resident = TRUE;
 }
 
-G_CONST_RETURN gchar*
+const gchar *
 g_module_error (void)
 {
   return g_static_private_get (&module_error_private);
@@ -648,7 +648,7 @@ g_module_symbol (GModule	*module,
   return !module_error;
 }
 
-G_CONST_RETURN gchar*
+const gchar *
 g_module_name (GModule *module)
 {
   g_return_val_if_fail (module != NULL, NULL);
@@ -663,7 +663,7 @@ g_module_name (GModule *module)
 
 #undef g_module_name
 
-G_CONST_RETURN gchar*
+const gchar *
 g_module_name (GModule *module)
 {
   g_return_val_if_fail (module != NULL, NULL);
