@@ -132,6 +132,10 @@ G_BEGIN_DECLS
  *  unmodified for the lifetime of the parameter. 
  *  Since 2.8
  * @G_PARAM_PRIVATE: internal
+ * @G_PARAM_DEPRECATED: the parameter is deprecated and will be removed
+ *  in a future version. A warning will be generated if it is used
+ *  while running with G_ENABLE_DIAGNOSTIC=1.
+ *  Since: 2.26
  * 
  * Through the #GParamFlags flag values, certain aspects of parameters
  * can be configured.
@@ -148,7 +152,9 @@ typedef enum
   G_PARAM_PRIVATE	      = G_PARAM_STATIC_NAME,
 #endif
   G_PARAM_STATIC_NICK	      = 1 << 6,
-  G_PARAM_STATIC_BLURB	      = 1 << 7
+  G_PARAM_STATIC_BLURB	      = 1 << 7,
+  /* User defined flags go up to 30 */
+  G_PARAM_DEPRECATED          = 1 << 31
 } GParamFlags;
 /**
  * G_PARAM_READWRITE:
@@ -175,11 +181,10 @@ typedef enum
  * G_PARAM_USER_SHIFT:
  * 
  * Minimum shift count to be used for user defined flags, to be stored in
- * #GParamSpec.flags.
+ * #GParamSpec.flags. The maximum allowed is 30 + G_PARAM_USER_SHIFT.
  */
 #define	G_PARAM_USER_SHIFT	(8)
 
-
 /* --- typedefs & structures --- */
 typedef struct _GParamSpec      GParamSpec;
 typedef struct _GParamSpecClass GParamSpecClass;
@@ -296,9 +301,9 @@ gboolean	g_param_value_convert		(GParamSpec    *pspec,
 gint		g_param_values_cmp		(GParamSpec    *pspec,
 						 const GValue  *value1,
 						 const GValue  *value2);
-G_CONST_RETURN gchar*	g_param_spec_get_name	(GParamSpec    *pspec);
-G_CONST_RETURN gchar*	g_param_spec_get_nick	(GParamSpec    *pspec);
-G_CONST_RETURN gchar*	g_param_spec_get_blurb	(GParamSpec    *pspec);
+const gchar *   g_param_spec_get_name           (GParamSpec    *pspec);
+const gchar *   g_param_spec_get_nick           (GParamSpec    *pspec);
+const gchar *   g_param_spec_get_blurb          (GParamSpec    *pspec);
 void            g_value_set_param               (GValue	       *value,
 						 GParamSpec    *param);
 GParamSpec*     g_value_get_param               (const GValue  *value);
